<?php

namespace SM\Service;

use SM\Factory\ProposalModelFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class ProposalModelService
{
    protected $ProposalModelFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, ProposalModelFactory $ProposalModelFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection               = $entityManager->getConnection();
        $this->ProposalModelFactory     = $ProposalModelFactory;
        $this->findService              = $findService;
        $this->Environment              = $Environment;
        $this->newCode                  = $newCode;
    }

    public function getProposalModel()
    {
        return $this->ProposalModelFactory->getProposalModel();
    }

    public function getTabEscopoFaturamentoProposal($params)
    {
        $params = array(
            "NRORCAMENTOMODELO"   => $params['NRORCAMENTOMODELO']
        );
        return $this->ProposalModelFactory->getTabEscopoFaturamentoProposal($params);
    }

    public function getTabComposicoes($params)
    {
        $params = array(
            "NRORCAMENTOMODELO"   => $params['NRORCAMENTOMODELO']
        );
    
        return $this->ProposalModelFactory->getTabComposicoes($params);
    }

    public function getTabItens($params)
    {
        $params = array(
            "NRORCAMENTOMODELO"   => $params['NRORCAMENTOMODELO']
        );
    
        return $this->ProposalModelFactory->getTabItens($params);
    }

    public function getTabPerfisAcessoProposal($params)
    {
        $params = array(
            "NRORCAMENTOMODELO"   => $params['NRORCAMENTOMODELO']
        );
        return $this->ProposalModelFactory->getTabPerfisAcessoProposal($params);
    }

    public function getTabSignatariosPadroesProposal($params)
    {
        $params = array(
            "NRORCAMENTOMODELO"   => $params['NRORCAMENTOMODELO']
        );
        return $this->ProposalModelFactory->getTabSignatariosPadroesProposal($params);
    }

    public function getServicoProduto()
    {
        return $this->ProposalModelFactory->getServicoProduto();
    }

    public function getTipoMedidaProposal()
    {
        return $this->ProposalModelFactory->getTipoMedidaProposal();
    }

    public function saveModalAddProposalModel($params)
    {
        $this->connection->beginTransaction();

        try {
            $this->ProposalModelFactory->editProposalModel(array(
                "NMMODCONTRATO"              => $params["NMMODCONTRATO"],
                "IDATIVO"                    => $params["IDATIVO"],
                "CDMODCONTRATO"              => $params["CDMODCONTRATO"],
                "IDGERAPROPOSTA"             => $params["IDGERAPROPOSTA"],
                "CDCENTCUST"                 => $params["CDCENTCUST"],
                "NRDIASVIGCONT"              => $params["NRDIASVIGCONT"],
                "IDRENOVACAOAUTOMATICA"      => $params["IDRENOVACAOAUTOMATICA"],
                "NRDIASPRORGCONT"            => $params["NRDIASPRORGCONT"],
                "IDBLOQALTDTINICONTRENEG"    => $params["IDBLOQALTDTINICONTRENEG"],
                "IDREAJUSTEAUTOMATICO"       => $params["IDREAJUSTEAUTOMATICO"],
                "NRDIASREAJAUTOMATICO"       => $params["NRDIASREAJAUTOMATICO"],
                "IDINTEGRACAOFM"             => $params["IDINTEGRACAOFM"],
                "IDINTEGRALOCAISFM"          => $params["IDINTEGRALOCAISFM"],
                "IDINTEGRAMEDICAOFM"         => $params["IDINTEGRAMEDICAOFM"],
                "IDINTEGRACAOHCM"        => $params["IDINTEGRACAOHCM"],
                "IDASSELETRDIGITAL"          => $params["IDASSELETRDIGITAL"],
                "NRTIPODOCCERTSIGN"          => $params["NRTIPODOCCERTSIGN"],
                "IDENVIADOCOPERRESPCONCASS"  => $params["IDENVIADOCOPERRESPCONCASS"],
                "NRORG"                      => $this->Environment->getNrOrg(),
                "CDOPERINCLUSAO"             => $this->Environment->getCdOperador(),
                "NRSEQMODCONTRA"             => $this->newCode->getCodeByTableName("FCT_MODELOCONTRATO", $this->Environment->getNrOrg(), 1, 10, true),
                "IDINTEGRACAOTECFOOD"        => $params["IDENVIADOCOPERRESPCONCASS"],
                "IDTIPOSERVINTTECFOOD"       => $params["IDTIPOSERVINTTECFOOD"],
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModel($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ProposalModelFactory->deleteProposalModel($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabEscopoFaturamento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ProposalModelFactory->deleteProposalModelTabEscopoFaturamento($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabComposicoes($parametros)
    {
        $this->connection->beginTransaction();

        try {
            if (isset($parametros["NRS"])) {
                $retorno = $this->findService->formatarArray($parametros["NRS"]);
                $retorno = str_replace("'", "", $retorno);
            } else {
                $retorno = $parametros;
            }

            $this->ProposalModelFactory->deleteProposalModelTabComposicoes($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabPerfisAcesso($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ProposalModelFactory->deleteProposalModelTabPerfisAcesso($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabSignatariosPadroes($parametros)
    {
        $this->connection->beginTransaction();

        try {
            if (isset($parametros["NRSIGNATPADRCONT"])) {
                $retorno = $this->findService->formatarArray($parametros["NRSIGNATPADRCONT"]);
            } else {
                $retorno = $parametros;
            }
            
            $this->ProposalModelFactory->deleteProposalModelTabSignatariosPadroes($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getEscopoDeFaturamentoProposal()
    {
        return $this->ProposalModelFactory->getEscopoDeFaturamentoProposal();
    }

    public function getModeloContratoProposal()
    {
        return $this->ProposalModelFactory->getModeloContratoProposal();
    }

    public function saveModalAddProposalModelTabEscopoFaturamento($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRRELACESCOPOMODELO = $this->newCode->getCodeByTableName("FCT_RELACESCOPOMODELO", $NRORG, 1, 10, true);

            $this->ProposalModelFactory->saveModalAddProposalModelTabEscopoFaturamento(array(
                "NRORCAMENTOMODELO"          => $params["NRORCAMENTOMODELO"],
                "NRTIPOORCAMENTO"            => $params["NRTIPOORCAMENTO"],
                "NRRELACESCOPOMODELO"        => $NRRELACESCOPOMODELO,
                "IDATIVO"                    => "S",
                "NRORG"                      => $NRORG,
                "CDOPERINCLUSAO"             => $this->Environment->getCdOperador()
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddProposalModelTabComposicoes($params)
    {
        $this->connection->beginTransaction();
        try {
            $CDOPERADOR = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();
            $NRRELORCAMODCOMP = $this->newCode->getCodeByTableName("FCT_RELORCAMODCOMPOSICAO", $NRORG, 1, 10, true);

            $this->ProposalModelFactory->saveModalAddProposalModelTabComposicoes(array(
                "NRRELORCAMODCOMP"           => $NRRELORCAMODCOMP,
                "NRORCAMENTOMODELO"          => $params["NRORCAMENTOMODELO"],
                "NRCOMPSERVICO"              => $params["NRCOMPSERVICO"],
                "IDATIVO"                    => "S",
                "NRORG"                      => $NRORG,
                "CDOPERINCLUSAO"             => $CDOPERADOR,
                "CDOPERULTATU"               => $CDOPERADOR,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddProposalModelTabPerfisAcesso($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRRELORCAMODPERFACESS = $this->newCode->getCodeByTableName("FCT_RELORCAMODPERFACESS", $NRORG, 1, 10, true);

            $this->ProposalModelFactory->saveModalAddProposalModelTabPerfisAcesso(array(
                "NRORCAMENTOMODELO"          => $params["NRORCAMENTOMODELO"],
                "IDATIVO"                    => "S",
                "NRPERFILACESSO"             => $params["NRPERFILACESSO"],
                "NRORG"                      => $NRORG,
                "CDOPERINCLUSAO"             => $this->Environment->getCdOperador(),
                "NRRELORCAMODPERFACESS"      => $NRRELORCAMODPERFACESS,
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddProposalModelTabSignatariosPadroes($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRSIGNATPADRCONT = $this->newCode->getCodeByTableName("FCT_SIGNATPADRMODCONT", $NRORG, 1, 10, true);
            $DSEMAILNOTSIGNENV = $this->findService->formatarArray($params["DSEMAILNOTSIGNENV"]);
            $DSEMAILNOTSIGNENV = str_replace("'", "", $DSEMAILNOTSIGNENV);
            $DSEMAILNOTSIGNENV = str_replace(",", "_", $DSEMAILNOTSIGNENV);

            $this->ProposalModelFactory->saveModalAddProposalModelTabSignatariosPadroes(array(
                "NRSIGNATPADRCONT"           => $NRSIGNATPADRCONT,
                "NRSEQMODCONTRA"             => $params["NRSEQMODCONTRA"],
                "IDTPSIGNATARIO"             => $params["IDTPSIGNATARIO"],
                "NRPARCNEGOCIO"              => $params["NRPARCNEGOCIO"],
                "IDTIPOASSINATURA"           => $params["IDTIPOASSINATURA"],
                "IDTESTEMUNHA"               => $params["IDTESTEMUNHA"],
                "NRORDENACAO"                => $params["NRORDENACAO"],
                "DSEMAILNOTSIGNENV"          => $DSEMAILNOTSIGNENV,
                "DSEMAILNOTIFOUTROS"         => $params["DSEMAILNOTIFOUTROS"],
                "NRORG"                      => $NRORG,
                "CDOPERINCLUSAO"             => $this->Environment->getCdOperador(),
                "IDATIVO"                    => "S"
            ));

            foreach ($params["CDOPERADOR"] as $CDOPERADOR) {
                $NRRELOPERNOTIFASSELE = $this->newCode->getCodeByTableName("FCT_RELACOPERNOTIFASSELE", $NRORG, 1, 10, true);
                $this->ProposalModelFactory->saveOperadoresSigPadrao(array(
                    "NRRELOPERNOTIFASSELE"   => $NRRELOPERNOTIFASSELE,
                    "NRSIGNATPADRCONT"       => $NRSIGNATPADRCONT,
                    "CDOPERADOR"             => $CDOPERADOR,
                    "NRORG"                  => $NRORG,
                    "CDOPERINCLUSAO"         => $this->Environment->getCdOperador(),
                    "IDATIVO"                => "S"
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getComposicoes()
    {
        return $this->ProposalModelFactory->getComposicoes();
    }

    public function getTipoDocCertsign()
    {
        return $this->ProposalModelFactory->getTipoDocCertsign();
    }

    public function getPerfisDeAcessoProposal()
    {
        return $this->ProposalModelFactory->getPerfisDeAcessoProposal();
    }

    public function getSignatariosProposalModel()
    {
        return $this->ProposalModelFactory->getSignatariosProposalModel();
    }

    public function getOperadoresProposalModel()
    {
        return $this->ProposalModelFactory->getOperadoresProposalModel();
    }

    public function getCpfSignatario($params)
    {
        $params = array(
            "NRPARCNEGOCIO"   => $params['signatario']
        );
        return $this->ProposalModelFactory->getCpfSignatario($params);
    }

    public function editProposalModelViewItens($params)
    {
        $this->connection->beginTransaction();
       
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            $this->deleteProposalModelTabSignatariosPadroes($params['NRSIGNATPADRCONT']);
            $this->saveModalAddProposalModelTabSignatariosPadroes($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editProposalModelTabSignatariosPadroes($params)
    {
        $this->connection->beginTransaction();
       
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            $this->deleteProposalModelTabSignatariosPadroes($params['NRSIGNATPADRCONT']);
            $this->saveModalAddProposalModelTabSignatariosPadroes($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editProposalModel($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
 
            $this->ProposalModelFactory->editProposalModel(array(
                "NRORCAMENTOMODELO"          => $params["NRORCAMENTOMODELO"],
                "IDATIVO"                    => $params["IDATIVO"],
                "IDBLOQALTERADADOS"          => $params["IDBLOQALTERADADOS"],
                "NMORCAMENTOMODELO"          => $params["NMORCAMENTOMODELO"],
                "QTDIASVALIDADE"             => $params["QTDIASVALIDADE"],
                "QTDIASCONTRATO"             => $params["QTDIASCONTRATO"],
                "IDUTILIZACOMP"              => $params["IDUTILIZACOMP"],
                "NRDIASCARENCIA"             => $params["NRDIASCARENCIA"],
                "IDOBRIGAOPORTUNIDADE"       => $params["IDOBRIGAOPORTUNIDADE"],
                "NRSEQMODCONTRA"             => $params["NRSEQMODCONTRA"],
                "CDMINEMORCA"                => $params["CDMINEMORCA"],
                "DSREFERENCIAORCA"           => $params["DSREFERENCIAORCA"],
                "DSOBSERVACAOORCA"           => $params["DSOBSERVACAOORCA"],
                "IDASSELETRDIGITAL"          => $params["IDASSELETRDIGITAL"],
                "IDENVIADOCCONCLUSAO"        => $params["IDENVIADOCCONCLUSAO"],
                "NRTIPODOCCERTSIGN"          => $params["NRTIPODOCCERTSIGN"],
                "IDENVIADOCOPERRESPCONCASS"  => $params["IDENVIADOCOPERRESPCONCASS"],
                "NRORG"                      => $NRORG,
                "CDOPERULTATU"               => $this->Environment->getCdOperador(),
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
