import { Utilities } from '@/util/Utilities';
import {
    Form, Modal, ModalService, Select, TextInput, Text, DialogService, Row, Footer, Textarea, LoadingService,
} from '@zeedhi/common';
import {
    Config,
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class AtividadeCdaController {
    public visibilidade: boolean = true;

    private geraDocumentoModal!: Modal;

    private modalAddRepresentante!: Modal;

    private modalCriarLancamentosRecorrentes!: Modal;

    public currentRow: any = {};

    public rowGeraDocumento: any = {};

    public campofilter: any = {};

    get visibilidadeDoFrame() {
        return !this.visibilidade;
    }

    public voltarPaginaInicial() {
        this.visibilidade = true;
        this.novaCda = false;
    }

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public visualizarAtividade({ component, row }: any) {
        component.selectedRows = [];
        this.validarPermissoesCda({ row });
        if (!this.disableCdaGrid) {
            this.visibilidade = false;
            this.currentRow = row;
            this.novaCda = false;
            this.montarInformacoesAtividade();
        }
    }

    public novaCda: boolean = false;

    public abrirNovaCda({ row }: any) {
        this.novaCda = true;
        const rowAux: any = (row === undefined) ? this.currentRow : row;
        this.validarPermissoesCda({ row: rowAux });
        /* Se a ação veio da action ele troca a visibilidade para exibir a tela interna */
        if (row !== undefined) {
            this.visibilidade = false;
            this.currentRow = row;
        }
        this.montarInformacoesAtividade();
    }

    public montarInformacoesAtividade() {
        Metadata.getInstance<Text>('titulo-agrupamento-info-atividade').text = `${this.currentRow.DATA} - ${this.currentRow.NMFUNC}`;
        const tipoLacamento = Metadata.getInstance<Select>('tipoLacamento-atividade-cda-tipolacamento');
        const idservprest = this.currentRow.IDSERVPREST;

        if (idservprest === null || idservprest === '' || this.novaCda) {
            tipoLacamento.value = '';
            tipoLacamento.disabled = false;
        } else {
            tipoLacamento.value = idservprest;
            tipoLacamento.disabled = true;
            this.ativaCampoPorTipoDeLacamento();
            this.carregarDadosAtividade();
        }
    }

    public ativaCampoPorTipoDeLacamento() {
        LoadingService.show();
        this.redefinirFormulario();

        const tipoLancamento = Metadata.getInstance<Select>('tipoLacamento-atividade-cda-tipolacamento').value;
        const rowParticipantes = Metadata.getInstance<Row>('row-participantes-lacamento-cda');
        const formParticipantes = Metadata.getInstance<Form>('form-participantes-lacamento-cda-editLacamento');
        const rowDescricao = Metadata.getInstance<Row>('row-descricao');
        const formDescricao = Metadata.getInstance<Form>('form-descricao');

        const cliente = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-cliente');
        const clienteOrig = Metadata.getInstance<Select>('cliente-orig');
        let cliPromise: Promise<any>;

        switch (tipoLancamento) {
            case 'ATIVIDADE_CLIENTE':
                cliPromise = cliente.datasource.setFilter({
                    CDFUNC: this.currentRow.CDFUNC,
                    DTSERVPREST: this.currentRow.DTSERVPREST,
                });

                clienteOrig.datasource.setFilter({
                    CDFUNC: this.currentRow.CDFUNC,
                    DTSERVPREST: this.currentRow.DTSERVPREST,
                });
            
                Metadata.getInstance<Row>('row-identificacao-atividade-cliente-cda').isVisible = true;
                Metadata.getInstance<Form>('form-identificacao-atividade-cliente-cda-editLacamento').isVisible = true;

                Metadata.getInstance<Row>('row-registro-de-horas-lancamento-cda').isVisible = true;
                Metadata.getInstance<Form>('form-registro-de-horas-cda-editLacamento').isVisible = true;
            
                Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-almoco').value = '00:00';
                Metadata.getInstance<TextInput>('registro-de-horas-cda-deslocamento-faturado').value = '00:00';
                Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-nao-faturadas').value = '00:00';

                rowParticipantes.isVisible = true;
                formParticipantes.isVisible = true;
                rowDescricao.isVisible = true;
                formDescricao.isVisible = true;

                this.montarGridHorasNaoFaturadas();
                this.montarGridParticipantes();

                if ((this.currentRow.IDSERVPREST === null || this.currentRow.IDSERVPREST === '') || this.novaCda) {
                    cliPromise.then(() => {
                        if (cliente.datasource.data.length === 1) {
                            cliente.value = cliente.datasource.data[0].CDCLIENTE;
                        }
                    });
                }
                break;
            case 'VISITA_PROSPECT':
                Metadata.getInstance<Row>('row-identificacao-visita-prospect-cda').isVisible = true;
                Metadata.getInstance<Form>('form-identificacao-visita-prospect-cda-editLacamento').isVisible = true;
                Metadata.getInstance<Row>('row-contato-lancamento-cda').isVisible = true;
                Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento').isVisible = true;
                break;
            case 'RELACIONAMENTO_CLIENTE':
                Metadata.getInstance<Row>('row-identificacao-relacionamento-cliente-cda').isVisible = true;
                Metadata.getInstance<Form>('form-identificacao-relacionamento-cliente-cda-editLacamento').isVisible = true;
                Metadata.getInstance<Row>('row-contato-lancamento-cda').isVisible = true;
                Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento').isVisible = true;
                break;
            case 'FALTA_JUSTIFICADA':
                Metadata.getInstance<Row>('row-justificativa-lancamento-cda').isVisible = true;
                Metadata.getInstance<Form>('form-justificativa-lacamento-cda-editLacamento').isVisible = true;
                break;
            case 'ATIVIDADE_INTERNA':
            case 'FERIAS':
            case 'FALTA_NAO_JUSTIFICADA':
                break;
            case 'AFASTAMENTO_INSS':
                Metadata.getInstance<Row>('row-justificativa-lancamento-cda').isVisible = true;
                Metadata.getInstance<Form>('form-justificativa-afastamentoinss-lacamento-cda-editLacamento').isVisible = true;
                break;
            default:
                break;
        }
        Metadata.getInstance<Footer>('footer-vizualizacao-atividade').isVisible = true;
        LoadingService.hide();
    }

    public async carregarDadosAtividade() {
        const tipoLancamento = Metadata.getInstance<Select>('tipoLacamento-atividade-cda-tipolacamento').value;
        const clienteOrig = Metadata.getInstance<Select>('cliente-orig');
        LoadingService.show();
        switch (tipoLancamento) {
            case 'ATIVIDADE_CLIENTE':
                Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-cliente').setValue(this.currentRow.CDCLIENTE, false);
                clienteOrig.setValue(this.currentRow.CDCLIENTE);
                this.setParamsCliente(this.currentRow.CDCLIENTE, this.currentRow.NRORG);
                Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-representante').setValue(this.currentRow.CDINTECLIE, false);
                Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-local').setValue(this.currentRow.NRLICENCAUSO, false);
                Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-projeto').setValue(this.currentRow.CDPROJETO, false);
                Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-proposta').setValue(this.currentRow.NRPROPOSFAS, false);
                this.setParametrosTipoAtividade(this.currentRow);
                Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-atividade').setValue(this.currentRow.CDTIPOSERV, false);

                Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-entrada').value = this.currentRow.DTENTRADA;
                Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-almoco').value = (this.currentRow.HRAUSENCIAALMOCO !== null) ? `${this.currentRow.HRAUSENCIAALMOCO.substring(0, 2)} ":" ${this.currentRow.HRAUSENCIAALMOCO.substring(2)}` : null;
                Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-saida').value = this.currentRow.DTSAIDA;
                Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-prestado').value = this.currentRow.QTHRTOTAL;
                Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-faturado').value = this.currentRow.QTHRATIVIDADEFAT;
                Metadata.getInstance<TextInput>('registro-de-horas-cda-deslocamento-faturado').value = this.currentRow.QTHRDESLOCAMENTOFAT;
                Metadata.getInstance<TextInput>('observacao-cda-horas-nao-faturadas').value = this.currentRow.DSJUSTHRNAOFATURADA;

                this.montarGridHorasNaoFaturadas();
                this.montarGridParticipantes();
                this.montarDadosDescricao();
                break;
            case 'VISITA_PROSPECT':
                Metadata.getInstance<Select>('identificacao-visita-prospect-cda-prospect').setValue(this.currentRow.CDPROSPECT, false);
                this.setParamsInterlocutorProspect(this.currentRow.CDPROSPECT, this.currentRow.NRPARCNEGOCIO);
                Metadata.getInstance<Select>('identificacao-visita-prospect-cda-intelocutor-prospect').setValue(this.currentRow.CDINTEPROS);
                break;
            case 'RELACIONAMENTO_CLIENTE':
                Metadata.getInstance<Select>('identificacao-relacionamento-cliente-cda').setValue(this.currentRow.CDCLIENTE, false);
                Metadata.getInstance<Select>('cliente-orig-relac').value = this.currentRow.CDCLIENTE;
                this.setParamsRelacCliente(this.currentRow.CDCLIENTE, this.currentRow.NRPARCNEGOCIO);
                Metadata.getInstance<Select>('identificacao-relacionamento-cliente-cda-intelocutor').setValue(this.currentRow.CDINTECLIE);
                break;
            case 'FALTA_JUSTIFICADA':
                Metadata.getInstance<Select>('justificativa-lacamento-cda').value = this.currentRow.NRMOTIVOEXCEFREQ;
                break;
            case 'ATIVIDADE_INTERNA':
            case 'FERIAS':
            case 'FALTA_NAO_JUSTIFICADA':
                break;
            case 'AFASTAMENTO_INSS':
                Metadata.getInstance<Select>('justificativa-lacamento-afastamentoinss-cda').value = this.currentRow.NRMOTIVOEXCEFREQ;
                break;
            default:
                break;
        }

        if (tipoLancamento === 'VISITA_PROSPECT' || tipoLancamento === 'RELACIONAMENTO_CLIENTE') {
            Metadata.getInstance<Textarea>('contato-lancamento-cda-descricao').value = this.currentRow.DSEVOLCONT;
            Metadata.getInstance<TextInput>('contato-lancamento-cda-telefone-intelocutor').value = this.currentRow.NRTELINTCLPR;
            Metadata.getInstance<TextInput>('contato-lancamento-cda-email-interlocutor').value = this.currentRow.DSEMAILINTCLPR;
            Metadata.getInstance<Select>('contato-lancamento-cda-prospect').setValue(this.currentRow.NRCONTATO);
            Metadata.getInstance<Select>('contato-lancamento-cda-intelocutor-motivo').setValue(this.currentRow.CDMOTICONT);
            Metadata.getInstance<Select>('contato-lancamento-cda-intelocutor-sistema').setValue(this.currentRow.CDSISTEMA);
            Metadata.getInstance<Select>('contato-lancamento-cda-motivo-evolucao').setValue(this.currentRow.CDMOTIEVOCON);
        }
        LoadingService.hide();
    }

    public montarDadosDescricao() {
        const desc = Metadata.getInstance<Textarea>('descricao-lacamento-cda-descricao');

        /* eslint-disable */
        LoadingService.show();
        Http.post('/getDescricaoCda', {
            CDEMPRESA: this.currentRow.CDEMPRESA,
            CDFUNC: this.currentRow.CDFUNC,
            CDCLIENTE: this.currentRow.CDCLIENTE,
            DTSERVPREST: this.currentRow.DTSERVPREST,
            NRSERVPREST: this.currentRow.NRSERVPREST,
        }).then((response: IDictionary) => {
            if (response.data.length > 0) {
                desc.value = response.data[0].DSSERVPREST;
            } else {
                desc.value = null;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
        /* eslint-enable */
    }

    public limparCampos() {
        Metadata.getInstance<Form>('form-participantes-lacamento-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-descricao').clearForm();
        Metadata.getInstance<Form>('form-identificacao-atividade-cliente-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-identificacao-visita-prospect-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-identificacao-relacionamento-cliente-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-justificativa-lacamento-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-justificativa-afastamentoinss-lacamento-cda-editLacamento').clearForm();
        Metadata.getInstance<Form>('form-registro-de-horas-cda-editLacamento').clearForm();
    }

    public redefinirFormulario() {
        Metadata.getInstance<Row>('row-participantes-lacamento-cda').isVisible = false;
        Metadata.getInstance<Form>('form-participantes-lacamento-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-descricao').isVisible = false;
        Metadata.getInstance<Form>('form-descricao').isVisible = false;
        Metadata.getInstance<Row>('row-identificacao-atividade-cliente-cda').isVisible = false;
        Metadata.getInstance<Form>('form-identificacao-atividade-cliente-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-identificacao-visita-prospect-cda').isVisible = false;
        Metadata.getInstance<Form>('form-identificacao-visita-prospect-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-contato-lancamento-cda').isVisible = false;
        Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-identificacao-relacionamento-cliente-cda').isVisible = false;
        Metadata.getInstance<Form>('form-identificacao-relacionamento-cliente-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-contato-lancamento-cda').isVisible = false;
        Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-justificativa-lancamento-cda').isVisible = false;
        Metadata.getInstance<Form>('form-justificativa-lacamento-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Form>('form-justificativa-afastamentoinss-lacamento-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Row>('row-registro-de-horas-lancamento-cda').isVisible = false;
        Metadata.getInstance<Form>('form-registro-de-horas-cda-editLacamento').isVisible = false;
        Metadata.getInstance<Footer>('footer-vizualizacao-atividade').isVisible = false;
        this.limparCampos();
    }

    public setParamsCliente(cdcliente: any, nrorg: any) {
        const representante = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-representante');
        const proposta = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-proposta');
        const localServico = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-local');
        const projeto = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-projeto');
        const atividade = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-atividade');
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');

        if (cdcliente) {
            projeto.datasource.setFilter({
                CDCLIENTE: cdcliente,
                CDOPERADOR: grid.datasource.currentRow.CDOPERADOR,
                NRORG: nrorg,
            });
            projeto.disabled = false;

            proposta.datasource.setFilter({
                CDCLIENTE: cdcliente,
                CDFUNC: grid.datasource.currentRow.CDFUNC,
            });
            proposta.disabled = false;

            representante.datasource.setFilter({
                CDCLIENTE: cdcliente,
            });
            representante.disabled = false;

            localServico.datasource.setFilter({
                CDCLIENTE: cdcliente,
            });
            localServico.disabled = false;
        } else {
            projeto.disabled = true;
            projeto.value = null;

            proposta.disabled = true;
            proposta.value = null;

            representante.disabled = true;
            representante.value = null;

            localServico.disabled = true;
            localServico.value = null;

            atividade.disabled = true;
            atividade.value = null;
        }
    }

    public onChangeCliente({ component }: any) {
        const representante = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-representante');
        const proposta = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-proposta');
        const localServico = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-local');
        const projeto = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-projeto');
        const atividade = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-atividade');
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');

        if (component.value) {
            // Projeto
            projeto.datasource.setFilter({
                CDCLIENTE: component.value,
                CDOPERADOR: grid.datasource.data[0].CDOPERADOR,
                NRORG: component.selectValue.NRORG,
            }).then(() => {
                if (projeto.datasource.data.length === 1) {
                    projeto.value = projeto.datasource.data[0].CDPROJETO;
                }
            });
            projeto.disabled = false;

            // Proposta
            proposta.datasource.setFilter({
                CDCLIENTE: component.value,
                CDFUNC: grid.datasource.data[0].CDFUNC,
            }).then(() => {
                if (proposta.datasource.data.length === 1) {
                    proposta.value = proposta.datasource.data[0].NRPROPOSFAS;
                }
            });
            proposta.disabled = false;
        
            // Representante
            representante.datasource.setFilter({
                CDCLIENTE: component.value,
            }).then(() => {
                if (representante.datasource.data.length === 1) {
                    representante.value = representante.datasource.data[0].CDINTECLIE;
                }
            });
            representante.disabled = false;
        
            // Local de Serviço
            localServico.datasource.setFilter({
                CDCLIENTE: component.value,
            }).then(() => {
                if (localServico.datasource.data.length === 1) {
                    localServico.value = localServico.datasource.data[0].CDCLIENTE;
                }
            });
            localServico.disabled = false;
        } else {
            projeto.disabled = true;
            projeto.value = null;

            proposta.disabled = true;
            proposta.value = null;

            representante.disabled = true;
            representante.value = null;

            localServico.disabled = true;
            localServico.value = null;

            atividade.disabled = true;
            atividade.value = null;
        }
    }

    public setParametrosTipoAtividade(row: any) {
        const atividade = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-atividade');

        if (row.NRPROPOSFAS) {
            atividade.datasource.setFilter({
                CDCLIENTE: row.CDCLIENTE,
                NRORG: row.NRORG,
                CDFUNC: row.CDFUNC,
                NRPROPOSFAS: row.NRPROPOSFAS,
                NRVERPROPFAS: row.NRVERPROPFAS,
                DTSERVPREST: row.DTSERVPREST,
            });
            atividade.disabled = false;
        } else {
            atividade.disabled = true;
            atividade.value = null;
        }
    }

    public onChangeTipoAtividade({ component }: any) {
        const atividade = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-atividade');

        if (component.value) {
            atividade.datasource.setFilter({
                CDCLIENTE: component.selectValue.CDCLIENTE,
                NRORG: component.selectValue.NRORG,
                CDFUNC: component.selectValue.CDFUNC,
                NRPROPOSFAS: component.selectValue.NRPROPOSFAS,
                NRVERPROPFAS: component.selectValue.NRVERPROPFAS,
                DTSERVPREST: this.currentRow.DTSERVPREST,
            }).then(() => {
                if (atividade.datasource.data.length === 1) {
                    atividade.value = atividade.datasource.data[0].CDTIPOSERV;
                }
            });
            atividade.disabled = false;
        } else {
            atividade.disabled = true;
            atividade.value = null;
        }
    }

    public setParamsInterlocutorProspect(cdProspect: any, nrparc:any) {
        const interlocutor = Metadata.getInstance<Select>('identificacao-visita-prospect-cda-intelocutor-prospect');
        const contato = Metadata.getInstance<Select>('contato-lancamento-cda-prospect');

        if (cdProspect) {
            interlocutor.datasource.setFilter({
                CDPROSPECT: cdProspect,
            });
            interlocutor.disabled = false;

            contato.datasource.setFilter({
                NRPARCNEGOCIO: nrparc,
            });
            contato.disabled = false;
        } else {
            interlocutor.disabled = true;
            interlocutor.value = null;
            contato.disabled = true;
            contato.value = null;
        }
    }

    public setParametroInterlocutorProspect({ component }: any) {
        const interlocutor = Metadata.getInstance<Select>('identificacao-visita-prospect-cda-intelocutor-prospect');
        const contato = Metadata.getInstance<Select>('contato-lancamento-cda-prospect');

        if (component.value) {
            interlocutor.datasource.setFilter({
                CDPROSPECT: component.value,
            });
            interlocutor.disabled = false;

            contato.datasource.setFilter({
                NRPARCNEGOCIO: component.selectValue.NRPARCNEGOCIO,
            });
            contato.disabled = false;
        } else {
            interlocutor.disabled = true;
            interlocutor.value = null;
            contato.disabled = true;
            contato.value = null;
        }
    }

    public setParamsRelacCliente(cdCliente: any, nrparc: any) {
        const interlocutor = Metadata.getInstance<Select>('identificacao-relacionamento-cliente-cda-intelocutor');
        const contato = Metadata.getInstance<Select>('contato-lancamento-cda-prospect');

        if (cdCliente.value) {
            interlocutor.datasource.setFilter({
                CDCLIENTE: cdCliente.value,
            });
            interlocutor.disabled = false;
        } else {
            interlocutor.disabled = true;
            interlocutor.value = null;
        }

        if (cdCliente.value) {
            contato.datasource.setFilter({
                NRPARCNEGOCIO: nrparc,
            });
            contato.disabled = false;
        } else {
            contato.disabled = true;
            contato.value = null;
        }
    }

    public onChangeRelacCliente({ component }: any) {
        const interlocutor = Metadata.getInstance<Select>('identificacao-relacionamento-cliente-cda-intelocutor');
        const contato = Metadata.getInstance<Select>('contato-lancamento-cda-prospect');

        if (component.value) {
            interlocutor.datasource.setFilter({
                CDCLIENTE: component.value,
            });
            interlocutor.disabled = false;
        } else {
            interlocutor.disabled = true;
            interlocutor.value = null;
        }

        if (component.value) {
            contato.datasource.setFilter({
                NRPARCNEGOCIO: component.selectValue.NRPARCNEGOCIO,
            });
            contato.disabled = false;
        } else {
            contato.disabled = true;
            contato.value = null;
        }
    }

    public setParametrosContato({ component }: IEventParam<Select>) {
        const email = Metadata.getInstance<TextInput>('contato-lancamento-cda-email-interlocutor');
        const telefone = Metadata.getInstance<TextInput>('contato-lancamento-cda-telefone-intelocutor');
        if (component.value) {
            telefone.value = component.selectValue.NRTEL1INPROS;
            email.value = component.selectValue.DSEMAILIPROS;
        } else {
            telefone.value = null;
            email.value = null;
        }
    }

    public setParametrosContatoCliente({ component }: IEventParam<Select>) {
        Metadata.getInstance<TextInput>('contato-lancamento-cda-telefone-intelocutor').value = (component.selectValue !== null) ? component.selectValue.NRTEL1INCLIE : null;
        Metadata.getInstance<TextInput>('contato-lancamento-cda-email-interlocutor').value = (component.selectValue !== null) ? component.selectValue.DSEMAILICLIE : null;
    }

    public arrayDeItens() {
        return [
            {
                DSJUSTHORASNAOFATUCDA: 'PRODUTO',
                QTHORASNAOFATU: '00:00',
                NRJUSTHORASNAOFATUCDA: 1,
                NRSERVPRESTJUSTHRNFAT: 0,
            },
            {
                DSJUSTHORASNAOFATUCDA: 'INFRA',
                QTHORASNAOFATU: '00:00',
                NRJUSTHORASNAOFATUCDA: 2,
                NRSERVPRESTJUSTHRNFAT: 0,
            },
            {
                DSJUSTHORASNAOFATUCDA: 'PROJETO',
                QTHORASNAOFATU: '00:00',
                NRJUSTHORASNAOFATUCDA: 3,
                NRSERVPRESTJUSTHRNFAT: 0,
            },
            {
                DSJUSTHORASNAOFATUCDA: 'RETRABALHO',
                QTHORASNAOFATU: '00:00',
                NRJUSTHORASNAOFATUCDA: 4,
                NRSERVPRESTJUSTHRNFAT: 0,
            },
            {
                DSJUSTHORASNAOFATUCDA: 'CAPACITAÇÃO',
                QTHORASNAOFATU: '00:00',
                NRJUSTHORASNAOFATUCDA: 5,
                NRSERVPRESTJUSTHRNFAT: 0,
            },
            {
                DSJUSTHORASNAOFATUCDA: 'DESLOCAMENTO',
                QTHORASNAOFATU: '00:00',
                NRJUSTHORASNAOFATUCDA: 6,
                NRSERVPRESTJUSTHRNFAT: 0,
            },
        ];
    }

    public montarGridHorasNaoFaturadas() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-horas-cda-horas-nao-faturadas');
        grid.datasource.updateData([]);

        if (this.currentRow.NRSERVPREST === '00000' || this.novaCda) {
            grid.datasource.updateData(this.arrayDeItens());
        } else {
            Http.post('/getHorasNaoFaturadas', {
                CDEMPRESA: this.currentRow.CDEMPRESA,
                CDFUNC: this.currentRow.CDFUNC,
                CDCLIENTE: this.currentRow.CDCLIENTE,
                DTSERVPREST: this.currentRow.DTSERVPREST,
                NRSERVPREST: this.currentRow.NRSERVPREST,
            }).then((response: IDictionary) => {
                if (response.data.data.length > 0) {
                    grid.datasource.updateData(response.data.data);
                } else {
                    grid.datasource.updateData(this.arrayDeItens());
                }
                this.preencherCampoHorasNaoFaturadas(grid);
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public stringHorasNFaturadas: string = '';

    public montarStringHorasNFaturadas({ component }: IEventParam<TekGrid>) {
        setTimeout(() => {
            const dados: any = component.datasource.data;
            let resultado = '';

            dados.forEach((p: any) => {
                resultado += `#${p.DSJUSTHORASNAOFATUCDA}#${this.corrigirFormatoHora(p.QTHORASNAOFATU)}`;
            });
            Metadata.getInstance<TextInput>('observacao-cda-horas-nao-faturadas').value = resultado;
        });
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = component.datasource.data.length + 1;
        component.addNewRow({ NRSERVPRESTPARTICIP: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSNOMEPARTICIP_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public montarGridParticipantes() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-participantes');

        if (this.novaCda) {
            grid.datasource.updateData([]);
        } else {
            Http.post('/getParticipantesCda', {
                CDEMPRESA: (this.currentRow.CDCLIENTE === null && this.currentRow.NRSERVPREST === '00000') ? null : this.currentRow.CDEMPRESA,
                CDFUNC: (this.currentRow.CDCLIENTE === null && this.currentRow.NRSERVPREST === '00000') ? null : this.currentRow.CDFUNC,
                CDCLIENTE: (this.currentRow.CDCLIENTE === null && this.currentRow.NRSERVPREST === '00000') ? null : this.currentRow.CDCLIENTE,
                DTSERVPREST: (this.currentRow.CDCLIENTE === null && this.currentRow.NRSERVPREST === '00000') ? null : this.currentRow.DTSERVPREST,
                NRSERVPREST: (this.currentRow.CDCLIENTE === null && this.currentRow.NRSERVPREST === '00000') ? null : this.currentRow.NRSERVPREST,
            }).then((response: IDictionary) => {
                if (response.data.data.length > 0) {
                    grid.datasource.updateData(response.data.data);
                } else {
                    grid.datasource.updateData([]);
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public dialogDeletarParticipantes({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'AtividadeCdaController.aposConfirmarDeleteParticipante');
    }

    public async aposConfirmarDeleteParticipante() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-participantes');
        LoadingService.show();

        Http.post('/deleteParticipantes', {
            NRSERVPRESTPARTICIP: grid.selectedRows.map((element: any) => element.NRSERVPRESTPARTICIP),
        }).then(() => {
            this.montarGridParticipantes();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public limparCampoHoraSaida() {
        Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-saida').value = null;
        Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-prestado').value = null;
    }

    public inserirAtividadeCDA() {
        const tipoLacamento = Metadata.getInstance<Select>('tipoLacamento-atividade-cda-tipolacamento');
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');
        const parametros = this.identificarFormularioValido({ component: tipoLacamento });
        const formTipoLancamento = Metadata.getInstance<Form>('form-tipoLacamento-atividade-cda-editLacamento');
        const gridParticip = Metadata.getInstance<TekGrid>('tek-grid-participantes');
        const gridHoras = Metadata.getInstance<TekGrid>('tek-grid-horas-cda-horas-nao-faturadas');

        this.calcularHorasServFaturado();
        if (parametros[0] && Utilities.validateForm(formTipoLancamento)) {
            if (!gridParticip.editing && !gridHoras.editing) {
                LoadingService.show();
                Http.post('/inserirAtividadeCDA', parametros[1]).then(() => {
                    grid.datasource.get();
                    Utilities.showSuccessMessage();
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    LoadingService.hide();
                    this.voltarPaginaInicial();
                });
            } else {
                Utilities.alertMessage('Existem grids em modo edição. Pressione "Salvar" ou "Cancelar" para prosseguir.', 'warning');
            }
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public identificarFormularioValido({ component }: IEventParam<Select>) {
        let formValido: any;
        const formCliente = Metadata.getInstance<Form>('form-identificacao-atividade-cliente-cda-editLacamento');
        const formContato = Metadata.getInstance<Form>('form-contato-lancamento-cda-editLacamento');
        const formProspect = Metadata.getInstance<Form>('form-identificacao-visita-prospect-cda-editLacamento');
        const formRelacionamento = Metadata.getInstance<Form>('form-identificacao-relacionamento-cliente-cda-editLacamento');
        const formFalta = Metadata.getInstance<Form>('form-justificativa-lacamento-cda-editLacamento');
        const formInss = Metadata.getInstance<Form>('form-justificativa-afastamentoinss-lacamento-cda-editLacamento');
        const formDescricao = Metadata.getInstance<Form>('form-descricao');
        const formHoras = Metadata.getInstance<Form>('form-registro-de-horas-cda-editLacamento');

        const justCda = Metadata.getInstance<Select>('justificativa-lacamento-cda').value;
        const justInss = Metadata.getInstance<Select>('justificativa-lacamento-afastamentoinss-cda').value;
        const cliente = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-cliente');
        const clienteRel = Metadata.getInstance<Select>('identificacao-relacionamento-cliente-cda');
        const proposta = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-proposta');
        const representante = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-representante');
        const interlocutor = Metadata.getInstance<Select>('identificacao-relacionamento-cliente-cda-intelocutor');
        const telefone = Metadata.getInstance<TextInput>('contato-lancamento-cda-telefone-intelocutor').value;
        const email = Metadata.getInstance<TextInput>('contato-lancamento-cda-email-interlocutor').value;
        const desContato = Metadata.getInstance<Textarea>('contato-lancamento-cda-descricao').value;
        const contato = Metadata.getInstance<Select>('contato-lancamento-cda-prospect').value;
        const cdmoticont = Metadata.getInstance<Select>('contato-lancamento-cda-intelocutor-motivo').value;
        const cdsistema = Metadata.getInstance<Select>('contato-lancamento-cda-intelocutor-sistema').value;
        const cdmotievocon = Metadata.getInstance<Select>('contato-lancamento-cda-motivo-evolucao').value;
        const cdProspect = Metadata.getInstance<Select>('identificacao-visita-prospect-cda-prospect');
        const clienteOrig = Metadata.getInstance<Select>('cliente-orig').value;
        const clientOrigRel = Metadata.getInstance<Select>('cliente-orig-relac').value;
        const tipoServ = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-atividade');
        // const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');

        const dados: any = ({
            CDEMPRESA: this.currentRow.CDEMPRESA,
            CDFUNC: this.currentRow.CDFUNC,
            DTSERVPREST: this.currentRow.DTSERVPREST,
            NRSERVPREST: (this.novaCda) ? '00000' : this.currentRow.NRSERVPREST,
            IDSERVPREST: component.value,
            DSREFERENCIA: this.currentRow.DSREFERENCIA,
            NRSEQEVOLCONT: this.currentRow.NRSEQEVOLCONT,
        });

        switch (component.value) {
            case 'ATIVIDADE_CLIENTE':
                // formulário válido
                formValido = (Utilities.validateForm(formCliente) && Utilities.validateForm(formDescricao) && Utilities.validateForm(formHoras));

                if (formValido) {
                    // parametros a ser enviado
                    dados.CDCLIENTE = cliente.value;
                    dados.CDINTECLIE = representante.value;
                    dados.NMINTCLIENTE = representante.selectValue.NMINTECLIE;
                    dados.NRLICENCAUSO = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-local').value;
                    dados.NRPROPOSFAS = proposta.value;
                    dados.NRVERPROPFAS = proposta.selectValue.NRVERPROPFAS;
                    dados.NRSERVCONT = tipoServ.selectValue.NRSERVCONT;
                    dados.CDPROJETO = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-projeto').value;
                    dados.CDTIPOSERV = tipoServ.value;
                    dados.NRPARCNEGOCIO = cliente.selectValue.NRPARCNEGOCIO;
                    dados.NRSEQAGESERV = tipoServ.selectValue.NRSEQAGESERV;
                    dados.DTENTRADA = Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-entrada').value;
                    dados.HRAUSENCIAALMOCO = Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-almoco').value;
                    dados.DTSAIDA = Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-saida').value;
                    dados.QTHRTOTAL = Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-prestado').value;
                    dados.QTHRDESLOCAMENTOFAT = Metadata.getInstance<TextInput>('registro-de-horas-cda-deslocamento-faturado').value;
                    dados.QTHRATIVIDADEFAT = Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-faturado').value;
                    dados.DESCSERVPRESS = Metadata.getInstance<Textarea>('descricao-lacamento-cda-descricao').value;
                    dados.PARTICIPANTES = Metadata.getInstance<TekGrid>('tek-grid-participantes').datasource.data;
                    dados.HORASNFATURADAS = Metadata.getInstance<TekGrid>('tek-grid-horas-cda-horas-nao-faturadas').datasource.data;
                    dados.DSJUSTHRNAOFATURADA = Metadata.getInstance<TextInput>('observacao-cda-horas-nao-faturadas').value;
                    dados.IDSITSERVPREST = 'P';
                    dados.CLIENTEORIG = clienteOrig;
                }
                break;
            case 'VISITA_PROSPECT':
                formValido = (Utilities.validateForm(formProspect) && Utilities.validateForm(formContato));

                if (formValido) {
                    dados.CDPROSPECT = cdProspect.value;
                    dados.CDINTEPROS = Metadata.getInstance<Select>('identificacao-visita-prospect-cda-intelocutor-prospect').value;
                    dados.NRSEQCONTATO = contato;
                    dados.CDMOTICONT = cdmoticont;
                    dados.CDSISTEMA = cdsistema;
                    dados.CDMOTIEVOCON = cdmotievocon;
                    dados.DSEMAILIPROS = email;
                    dados.NRTEL1INPROS = telefone;
                    dados.DSEVOLCONT = desContato;
                    dados.NRPARCNEGOCIO = cdProspect.selectValue.NRPARCNEGOCIO;
                }
                break;
            case 'RELACIONAMENTO_CLIENTE':
                formValido = (Utilities.validateForm(formRelacionamento) && Utilities.validateForm(formContato));

                if (formValido) {
                    dados.CDCLIENTE = clienteRel.value;
                    dados.CDINTECLIE = interlocutor.value;
                    dados.NMINTCLIENTE = interlocutor.selectValue.NMINTECLIE;
                    dados.NRPARCNEGOCIO = clienteRel.selectValue.NRPARCNEGOCIO;
                    // dados.NRSEQAGESERV = clienteRel.selectValue.NRSEQAGESERV;
                    dados.DSEVOLCONT = desContato;
                    dados.DTHREVOLCONT = null;
                    dados.DSEMAILICLIE = email;
                    dados.NRTEL1INCLIE = telefone;
                    dados.NRSEQCONTATO = contato;
                    dados.CDMOTICONT = cdmoticont;
                    dados.CDSISTEMA = cdsistema;
                    dados.CDMOTIEVOCON = cdmotievocon;
                    dados.CLIENTEORIG = clientOrigRel;
                }
                break;
            case 'FALTA_JUSTIFICADA':
                formValido = (Utilities.validateForm(formFalta));

                if (formValido) {
                    dados.NRMOTIVOEXCEFREQ = justCda;
                }
                break;
            case 'AFASTAMENTO_INSS':
                formValido = (Utilities.validateForm(formInss));

                if (formValido) {
                    dados.NRMOTIVOEXCEFREQ = justInss;
                }
                break;
            case 'ATIVIDADE_INTERNA':
                formValido = true;
                break;
            case 'FERIAS':
                formValido = true;
                break;
            case 'FALTA_NAO_JUSTIFICADA':
                formValido = true;
                break;
            default:
                break;
        }

        return [formValido, dados];
    }

    public corrigirFormatoHora(hora: any) {
        // Remover quaisquer caracteres não numéricos
        let h = hora.replace(/\D/g, '');

        // Se a hora tiver menos de 4 dígitos, adicionar "0" à esquerda para obter um formato de 4 dígitos
        if (h.length < 4) {
            h = h.padStart(4, '0');
        }

        // Inserir dois pontos no formato HH:mm
        return `${h.slice(0, 2)}:${h.slice(2)}`;
    }

    public calcularHorasServPrest() {
        const inputSaida = Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-saida');
        const inputAlmoco = Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-almoco');
        const inputEntrada = Metadata.getInstance<TextInput>('registro-de-horas-cda-hora-entrada');
        const horasPrestadas = Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-prestado');
    
        if (inputSaida.value && inputEntrada.value) {
            horasPrestadas.value = this.calcularHoraDeServico(inputEntrada.value, inputSaida.value, ((inputAlmoco.value) ? inputAlmoco.value : null));
        } else {
            horasPrestadas.value = null;
        }
        this.calcularHorasServFaturado();
    }

    public calcularHoraDeServico(horaE: string, horaS: string, horarioA?: string): string | null {
        // Corrija o formato das horas de entrada e saída se necessário
        const horaEntrada = this.corrigirFormatoHora(horaE);
        const horaSaida = this.corrigirFormatoHora(horaS);

        // Continuar com o cálculo da duração do serviço
        const [entradaHoras, entradaMinutos] = horaEntrada.split(':').map(Number);
        const [saidaHoras, saidaMinutos] = horaSaida.split(':').map(Number);

        const entradaTotalMinutos = entradaHoras * 60 + entradaMinutos;
        const saidaTotalMinutos = saidaHoras * 60 + saidaMinutos;

        let horarioAlmocoTotalMinutos = 0;

        // Se o horário de almoço for fornecido, corrija o formato
        // Se o horário de almoço for fornecido, calcule sua duração
        if (horarioA) {
            const horarioAlmoco = this.corrigirFormatoHora(horarioA);
            const [almocoHoras, almocoMinutos] = horarioAlmoco.split(':').map(Number);
            horarioAlmocoTotalMinutos = almocoHoras * 60 + almocoMinutos;
        }

        // Calcule a duração do serviço (subtraindo o horário de almoço, se aplicável)
        const duracaoTotalMinutos = saidaTotalMinutos - entradaTotalMinutos - horarioAlmocoTotalMinutos;

        const horas = Math.floor(duracaoTotalMinutos / 60);
        const minutos = duracaoTotalMinutos % 60;

        return `${horas.toString().padStart(2, '0')}:${minutos.toString().padStart(2, '0')}`;
    }

    public calcularHorasServFaturado() {
        const horasServPrest = Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-prestado');
        const deslocamentoFaturado = Metadata.getInstance<TextInput>('registro-de-horas-cda-deslocamento-faturado');
        const inputFaturado = Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-servico-faturado');
        const horasNaoFaturadas = Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-nao-faturadas');

        inputFaturado.value = this.calcularHorasFaturadas(horasServPrest.value, deslocamentoFaturado.value, horasNaoFaturadas.value);
    }

    public calcularHorasFaturadas(horaS: string, horaD: string, horasN: string): string | null {
        // Corrigir o formato das horas de serviço prestado e deslocamento, se necessário
        if (horaS) {
            const horaServicoPrestado = this.corrigirFormatoHora(horaS);
            const horaDeslocamento = (horaD === null) ? this.corrigirFormatoHora('00:00') : this.corrigirFormatoHora(horaD);
            const horaNaoFaturada = (horasN === null) ? this.corrigirFormatoHora('00:00') : this.corrigirFormatoHora(horasN);
    
            // Continuar com o cálculo das horas faturadas
            const [servicoPrestadoHoras, servicoPrestadoMinutos] = horaServicoPrestado.split(':').map(Number);
            const [deslocamentoHoras, deslocamentoMinutos] = horaDeslocamento.split(':').map(Number);
            const [naoFaturadasHoras, naoFaturadasMinutos] = horaNaoFaturada.split(':').map(Number);
    
            const servicoPrestadoTotalMinutos = servicoPrestadoHoras * 60 + servicoPrestadoMinutos;
            const deslocamentoTotalMinutos = deslocamentoHoras * 60 + deslocamentoMinutos;
            const naoFaturadasTotalMinutos = naoFaturadasHoras * 60 + naoFaturadasMinutos;

            const horasFaturadasTotalMinutos = servicoPrestadoTotalMinutos + deslocamentoTotalMinutos - naoFaturadasTotalMinutos;
    
            if (horasFaturadasTotalMinutos < 0) {
                const message: string = 'O total de "Horas de serviço não faturadas" não pode ser maior que "Horas de serviço prestado" + "Deslocamento Faturado"';
                Utilities.alertMessage(message, 'error');
                throw new Error(message);
            }
            const horas = Math.floor(horasFaturadasTotalMinutos / 60);
            const minutos = horasFaturadasTotalMinutos % 60;
    
            return `${horas.toString().padStart(2, '0')}:${minutos.toString().padStart(2, '0')}`;
        }
        return null;
    }

    public calcularHorasNaoFaturadas({ event, component }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            // Mesclando o array de data com o das linhas editadas, para que tanto as horas já existentes quanto as editadas sejam consideradas
            this.preencherCampoHorasNaoFaturadas(component);
            this.calcularHorasServFaturado();
            component.saveEditedRows();
        }
        this.montarStringHorasNFaturadas({ component });
    }

    public preencherCampoHorasNaoFaturadas(component: any) {
        const result = this.mesclarDatasources(component.datasource.data, component.getEditedRows());
        Metadata.getInstance<TextInput>('registro-de-horas-cda-horas-nao-faturadas').value = this.somarHorasDeServicoNaoFaturado(result);
    }

    public mesclarDatasources(data1: any, data2: any) {
        const mergedData = new Map();
        // Mesclar os dados do primeiro conjunto
        data1.forEach((item: any) => {
            mergedData.set(item.DSJUSTHORASNAOFATUCDA, item);
        });

        // Mesclar os dados do segundo conjunto, sobrescrevendo os existentes se houver duplicatas
        data2.forEach((item: any) => {
            mergedData.set(item.DSJUSTHORASNAOFATUCDA, item);
        });

        return Array.from(mergedData.values());
    }

    public somarHorasDeServicoNaoFaturado(dados: any[]): string {
        let totalHoras = '00:00';

        dados.forEach((item) => {
            const horaFormatada = this.corrigirFormatoHora(item.QTHORASNAOFATU);
            const horaSeparada = horaFormatada.split(':');
            const horas = parseInt(horaSeparada[0], 10);
            const minutos = parseInt(horaSeparada[1], 10);

            totalHoras = this.adicionarHoras(totalHoras, horas, minutos);
        });

        return totalHoras;
    }

    public adicionarHoras(horaExistente: string, horasParaAdicionar: number, minutosParaAdicionar: number): string {
        const [horaExistenteHoras, horaExistenteMinutos] = horaExistente.split(':');

        let horas = parseInt(horaExistenteHoras, 10);
        let minutos = parseInt(horaExistenteMinutos, 10);

        horas += horasParaAdicionar;
        minutos += minutosParaAdicionar;

        horas += Math.floor(minutos / 60);
        minutos %= 60;

        return `${String(horas).padStart(2, '0')}:${String(minutos).padStart(2, '0')}`;
    }

    public existenciaCda({ row }: any) {
        if (row.CDCLIENTE === null || row.NRSERVPREST === '00000') {
            return false;
        }
        return true;
    }

    private grupoOper: boolean = false;

    public async getPermissaoFunc() {
        await Http.get('/getPermissaoOperadorCda').then((response: any) => {
            if (response.data.length > 0) {
                // Verifique se pelo menos um objeto tem IDAUTGRUPOP igual a "ACSA" ou "DPAP"
                this.grupoOper = response.data.some((elemento: any) => elemento.IDAUTGRUPOP === 'ACSA' || elemento.IDAUTGRUPOP === 'DPAP');
            } else {
                this.grupoOper = false;
            }
        });
    }

    public disableCdaFields: boolean = false;

    public disableCdaGrid: boolean = false;

    public validarPermissoesCda({ row }: any) {
        const existeCda = this.existenciaCda({ row });
        const dataAtual = new Date();
        const qtdDiasData = Utilities.getQtdeDiasEntre(row.DTSERVPREST, dataAtual);
        const periodo = Metadata.getInstance<Select>('formFilterCda-PERIODOAPURAPONTO').datasource.currentRow.STATUS;
        this.periodoAtual = periodo;

        // Desabilita Form_View e todos os campos qdo não tiver permissão e expirado o prazo limite de 7 dias e não existir Cda lançada.
        if (!(this.grupoOper) && (qtdDiasData > 7) && !existeCda) {
            this.disableCdaGrid = true;
            this.disableCdaFields = true;
            this.criaRecorrencia = false;
        } else if (!(this.grupoOper) && (qtdDiasData > 7) && existeCda) { // Se não tiver permissão e expirado o prazo limite de 7 dia, mas existir Cda lançada habilita a action, mas irá desabilitar os campos.
            this.disableCdaGrid = false;
            this.disableCdaFields = true;
            this.criaRecorrencia = false;
        } else if (periodo !== '(Em aberto)' && existeCda) {
            this.disableCdaGrid = false;
            this.disableCdaFields = true;
            this.criaRecorrencia = false;
        } else if (periodo !== '(Em aberto)' && !existeCda) {
            this.disableCdaGrid = true;
            this.disableCdaFields = true;
            this.criaRecorrencia = false;
        } else { // Caso contrário habilita a action e não desabilita os campos.
            this.disableCdaGrid = false;
            this.criaRecorrencia = true;
            if (row.IDSITSERVPREST !== 'P' && this.novaCda) {
                this.disableCdaFields = false;
            } else if (row.IDSITSERVPREST !== 'P' && row.IDSITSERVPREST !== null) {
                this.disableCdaFields = true;
            } else {
                this.disableCdaFields = false;
            }
        }
    }

    /* disableCdaGrid = se o grid está habilitado ou não
       IDDIAUTIL define o dia útil, com base nisso o grid recebe um background cinza para dias não úteis.
    */
    public styleColunaCda({ row }: IEventParam<TekGrid>) {
        if (this.disableCdaGrid) {
            if (row.IDDIAUTIL === 'N') {
                return 'coluna-cda-cinza-com-background';
            }
            return 'coluna-cda-cinza';
        }
        if (row.IDDIAUTIL === 'N') {
            return 'coluna-cda-azul-com-background';
        }
        return 'coluna-cda-azul';
    }

    public rowStyleConditions(row: any) {
        if (row.IDDIAUTIL === 'N') {
            if (row.IDSITSERVPREST !== 'P') {
                return {
                    cssClass: 'cda-confirmada-background',
                };
            }

            return {
                cssClass: 'background-util-day-grey',
            };
        }

        if (row.IDSITSERVPREST !== 'P') {
            return {
                cssClass: 'cda-confirmada',
            };
        }

        return null;
    }

    public visibleConfirmCda: boolean = false;

    public visibleCorrigCda: boolean = false;

    public visibleImprCda: boolean = false;

    /* Se a CDA estiver confirmada as informações do grid estarão em negrito
       Caso ela esteja bloqueada e com apenas visualização, os dados estarão cinza.
    */
    public styleColunaCdaComDados({ row }: IEventParam<TekGrid>) {
        if (row.IDSITSERVPREST !== 'P') {
            return 'texto-cinza-negrito-cda-confirmada';
        }
        if (!(row.IDSITSERVPREST !== 'P') && this.disableCdaFields) {
            return 'texto-cinza-cda-visualizacao';
        }
        return null;
    }

    public validarAcoesAdicionarCda({ component, row }: any) {
        const rowAux: any = (row === undefined) ? this.currentRow : row;
        let message: string = '';

        if (component.label === 'Adicionar Nova CDA') {
            this.novaCda = true;
            this.validarPermissoesCda({ row: rowAux });
            if (this.disableCdaFields || row.IDSERVPREST === 'FALTA_JUSTIFICADA' || row.IDSERVPREST === 'FALTA_NAO_JUSTIFICADA'
            || row.IDSERVPREST === 'AFASTAMENTO_INSS' || row.IDSERVPREST === 'FERIAS') {
                Utilities.showDialogMessage('warning', 'Acesso Bloqueado', 'Não é permitido adicionar uma nova CDA neste dia, pois expirou o prazo de 7 dias e/ou, o tipo de lançamento não permite!');
            } else {
                this.abrirNovaCda({ row: rowAux });
            }
        } else if (component.label === 'Confirmar CDA') {
            this.novaCda = false;
            this.validarPermissoesCda({ row: rowAux });

            if (rowAux.IDSERVPREST !== 'ATIVIDADE_CLIENTE') {
                if (rowAux.IDSERVPREST !== null) {
                    message = `Não é permitido confirmar uma CDA do tipo: ${rowAux.IDSERVPREST}`;
                } else {
                    message = 'Não é permitido confirmar uma CDA vazia.';
                }
                Utilities.showDialogMessage('warning', 'Acesso Bloqueado', message);
            } else {
                this.validarConfirmacaoCda({ row: rowAux });
            }
        } else if (component.label === 'Corrigir CDA') {
            this.novaCda = false;
            this.validarPermissoesCda({ row: rowAux });

            if (rowAux.IDSERVPREST !== 'ATIVIDADE_CLIENTE') {
                if (rowAux.IDSERVPREST !== null) {
                    message = `Não é permitido corrigir uma CDA do tipo: ${rowAux.IDSERVPREST}`;
                } else {
                    message = 'Não é permitido corrigir uma CDA vazia.';
                }
                Utilities.showDialogMessage('warning', 'Acesso Bloqueado', message);
            } else {
                this.validarCorrecaoCda({ row: rowAux });
            }
        }
    }

    public validarConfirmacaoCda({ row }: any) {
        const periodo = Metadata.getInstance<Select>('formFilterCda-PERIODOAPURAPONTO').datasource.currentRow.STATUS;
        const changeEnabled: any = (((row.IDSITSERVPREST === '') || (row.IDSITSERVPREST === 'P'))
            && (periodo === '(Em aberto)')
            && (row.IDDESCRICAOOK === 'S')
            && ((row.DTENTRADA !== '' && row.DTENTRADA !== null)));

        if (row.NRSERVPREST !== null && row.NRSERVPREST !== '00000') {
            if (changeEnabled) {
                this.confirmarOuCorrigirCda({ component: null }, 'confirmar', row);
            } else {
                Utilities.showDialogMessage('warning', 'Acesso Bloqueado', 'Não é possível confirmar esta CDA devido às seguintes opções: podem ter excedido o prazo de 7 dias, você pode não ter permissão, ela(s) pode(m) já ter sido confirmada(s) ou existem campos obrigatórios não preenchidos.');
            }
        }
    }

    public validarCorrecaoCda({ row }: any) {
        const periodo = Metadata.getInstance<Select>('formFilterCda-PERIODOAPURAPONTO').datasource.currentRow.STATUS;

        if (row.NRSERVPREST !== null && row.NRSERVPREST !== '00000') {
            if (this.grupoOper && (row.IDSITSERVPREST === 'C') && (periodo === '(Em aberto)')) {
                this.confirmarOuCorrigirCda({ component: null }, 'corrigir', row);
            } else {
                Utilities.showDialogMessage('warning', 'Acesso Bloqueado', 'Você não possui permissão para corrigir esta CDA!');
            }
        }
    }

    public confirmarOuCorrigirCda({ component }: any, acao: string, row: any) {
        LoadingService.show();
        const rowAux = (row === undefined) ? this.currentRow : row;
        const acaoAux = (acao === undefined) ? component.name.split('_')[0] : acao;
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');
        Http.post('/confirmaOuCorrigeCda', {
            CDEMPRESA: rowAux.CDEMPRESA,
            CDFUNC: rowAux.CDFUNC,
            CDCLIENTE: rowAux.CDCLIENTE,
            DTSERVPREST: rowAux.DTSERVPREST,
            NRSERVPREST: rowAux.NRSERVPREST,
            ACAO: acaoAux,
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
            grid.selectedRows = [];
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public conditionsGridsInternos() {
        return !this.disableCdaFields;
    }

    get conditionsAdd() {
        return this.conditionsGridsInternos();
    }

    get conditionsDelete() {
        if (this.disableCdaFields) {
            return 'none';
        }
        return 'selection';
    }

    public validarDelete({ row }: any) {
        const periodo = Metadata.getInstance<Select>('formFilterCda-PERIODOAPURAPONTO').datasource.currentRow.STATUS;
        let changeEnabled = (((row.IDSITSERVPREST === '') || (row.IDSITSERVPREST === 'P')) && (periodo === '(Em aberto)'));
        this.validarPermissoesCda({ row });

        if (this.disableCdaFields) {
            changeEnabled = false;
        }

        if (row.NRSERVPREST !== null && row.NRSERVPREST !== '00000') {
            // se o periodo ta aberto e se o idsit é P ou vazio.
            if (changeEnabled) {
                return true;
            }
            return false;
        }

        return false;
    }

    public acaoDelete({ component, event }: any) {
        event.preventDefault();
        const dtBloq: any = [];
        const dtLib: any = [];
        let message: any = '';
        let disableButton: boolean = false;

        component.selectedRows.forEach((element: any) => {
            if (this.validarDelete({ row: element })) {
                dtLib.push(element.DTSERVPREST);
                this.cdaAExcluir.push(element);
            } else {
                dtBloq.push(element.DTSERVPREST);
            }
        });

        if (dtBloq.length > 0 && dtLib.length > 0) {
            const blockedDates = dtBloq.map((element: any) => `${element}`).join(', ');
            const allowedDates = dtLib.map((element: any) => `${element}`).join(', ');

            message = `A(s) CDA(s) referente(s) à(s) data(s) ${blockedDates} não pode(m) ser excluída(s) devido às seguintes opções: podem ter excedido o prazo de 7 dias, você pode não ter permissão, elas podem já ter sido confirmadas ou serem CDAs vazias.<br><br>A(s) CDA(s) referente(s) a(s) data(s) ${allowedDates} pode(m) ser(ão) excluída(s). Esta ação, depois de realizada, não poderá ser desfeita. Deseja continuar?`;
            disableButton = false;
        } else if (dtBloq.length > 0) {
            const blockedDates = dtBloq.map((element: any) => `${element}`).join(', ');

            message = `A(s) CDA(s) referente(s) à(s) data(s) ${blockedDates} não pode(m) ser excluída(s) devido às seguintes opções: podem ter excedido o prazo de 7 dias, você pode não ter permissão, elas podem já ter sido confirmadas ou serem CDAs vazias.`;
            disableButton = true;
        } else if (dtLib.length > 0) {
            const allowedDates = dtLib.map((element: any) => `${element}`).join(', ');

            message = `A(s) CDA(s) referente(s) a(s) data(s) ${allowedDates} pode(m) ser(ão) excluída(s). Esta ação, depois de realizada, não poderá ser desfeita. Deseja continuar?`;
            disableButton = false;
        }

        Utilities.showDialogConfirm('Atenção!', event, 'AtividadeCdaController.deletarCda', message, disableButton);
    }

    private cdaAExcluir: any = [];

    public deletarCda() {
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');

        LoadingService.show();
        Http.post('/deletarCda', this.cdaAExcluir).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            this.cdaAExcluir = [];
            grid.selectedRows = [];
            DialogService.hide();
            LoadingService.hide();
        });
    }

    public disabledGerarDoc: boolean = true;

    public habilitarActionDoc({ component }: any) {
        /* eslint-disable */
        this.disabledGerarDoc = !(component.selectedRows.some((element: any) => {
            return (element.IDSERVPREST === 'ATIVIDADE_CLIENTE' && element.IDSITSERVPREST !== 'P');
        }));
        /* eslint-enable */
    }

    public desabilitarActionDoc() {
        this.disabledGerarDoc = true;
    }

    public gerarModeloDocumento() {
        LoadingService.show();
        this.verificaIdiomaDoc().then((response: any) => {
            if (response.data.data.length > 0) {
                Utilities.showSuccessMessage();
                this.validarGeracaoDocAction(response.data.data[0].CDIDIOM);
            } else {
                Utilities.alertMessage('Não existe documento padrão para o idioma selecionado.', 'error');
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            this.fecharModalGeraDoc();
        });
    }

    public async verificaIdiomaDoc() {
        return new Promise((resolve, reject) => {
            Http.get(`/getModeloDocumentoPadraoCda?CDIDIOM=${Metadata.getInstance<Select>('FORM-GERA-CDA-CDIDIOM').value}`).then((response: IDictionary) => {
                resolve(response);
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
                reject(error);
            });
        });
    }

    public async validarGeracaoDocAction(cdIdioma: string) {
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');
        const cdaBloq: any = [];
        const cdaLib: any = [];

        const rows = grid.selectedRows;
        rows.sort((element: any, elementB: any) => {
            if (element.DTSERVPRESTORDER < elementB.DTSERVPRESTORDER) {
                return -1;
            }
            if (element.DTSERVPRESTORDER > elementB.DTSERVPRESTORDER) {
                return 1;
            }
            // names must be equal
            return 0;
        });

        rows.forEach((element: any) => {
            if (element.IDSERVPREST === 'ATIVIDADE_CLIENTE' && element.IDSITSERVPREST !== 'P') {
                cdaLib.push(element);
            } else {
                cdaBloq.push(element.DTSERVPREST);
            }
        });
    
        const response = await Http.post('/gerarDocumentoCda', {
            CDA: cdaLib,
            IDIOMA: cdIdioma,
        });
    
        if (response.data.error) {
            Utilities.showErrorMessage(response.data.error);
        } else {
            window.open(Config.endPoint + response.data.link);
        }
            
        this.disabledGerarDoc = true;
        grid.selectedRows = [];
        this.fecharModalGeraDoc();
        LoadingService.hide();

        if (cdaBloq.length > 0) {
            Utilities.showDialogMessage('warning', 'Atenção!', `A(s) CDA(s) referente(s) à(s) data(s) ${cdaBloq.map((elemento: any) => `${elemento}`).join(', ')} não pode(m) ser impressa(s)`);
        }
    }

    public fecharModalGeraDoc() {
        this.geraDocumentoModal.hide();
        this.geraDocumentoModal?.destroy();
    }
    
    public async abrirModalGeraDoc() {
        this.geraDocumentoModal = await ModalService.createFromJson('sm_cda/Modal/gerar_documento_cda', true);
        await this.geraDocumentoModal.show();
    }

    public modoEdicao({ component }: any) {
        component.editing = true;
    }

    public setIdioma({ component }: any) {
        setTimeout(() => {
            component.setValue(component.datasource.data.find((element: any) => element.NMIDIOMA === 'Português'));
        }, 1000);
    }

    public setFocus({ component }: any) {
        setTimeout(() => {
            component.setFocus();
        });
    }

    public fecharModalRepresentante() {
        this.modalAddRepresentante.hide();
        this.modalAddRepresentante?.destroy();
    }
    
    public async abrirRepresentante() {
        this.modalAddRepresentante?.destroy();
        this.modalAddRepresentante = await ModalService.createFromJson('sm_cda/Modal/modal_add_representante', true);
        const cliente = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-cliente').selectValue;

        if (cliente !== null) {
            await this.modalAddRepresentante.show();
            Metadata.getInstance<Text>('text-information').text = `Lembre-se, este Represente será associado ao Cliente: ${cliente.NMRAZSOCCLIE}`;
        } else {
            Utilities.alertMessage('Selecione um Cliente para prosseguir', 'warning');
        }
    }

    public salvarRepresentante() {
        const form = Metadata.getInstance<Form>('form_modal_add_representante');
        const representante = Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-representante');

        if (Utilities.validateForm(form)) {
            Http.post('/inserirRepresentante', {
                CDCLIENTE: Metadata.getInstance<Select>('identificacao-atividade-cliente-cda-cliente').value,
                NMINTECLIE: Metadata.getInstance<TextInput>('nome-representante').value,
            }).then(() => {
                Utilities.showSuccessMessage();
                representante.datasource.get();
                this.fecharModalRepresentante();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalGeraDoc();
            });
        }
    }

    public validaRowsSelecionadasAbreModalLancRec() {
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');

        if (grid.selectedRows.length > 0) {
            const dtInvalida = this.verificaTodasRowsInvalidas(grid);

            if (dtInvalida.length > 0) {
                Utilities.showDialogMessage('warning', 'Aviso', `A(s) CDA(s) referente(s) à(s) data(s) ${dtInvalida.map((element: any) => `${element}`).join(', ')} não permite(m) a criação de lançamentos recorrentes.<br><br> Certifique-se de que o período selecionado não possua lançamentos como: Férias, Faltas ou Afastamentos.`);
            } else {
                this.abrirModalCriarLancRec();
            }
        } else {
            Utilities.showDialogMessage('warning', 'Aviso', 'Selecione pelo menos um registro!');
        }
    }

    public verificaTodasRowsInvalidas(grid: TekGrid) {
        const dtBloq: any = [];

        grid.selectedRows.forEach((row) => {
            this.validarPermissoesCda({ row });
            if (this.criaRecorrencia && (row.IDSERVPREST === 'FALTA_JUSTIFICADA' || row.IDSERVPREST === 'FALTA_NAO_JUSTIFICADA'
            || row.IDSERVPREST === 'AFASTAMENTO_INSS' || row.IDSERVPREST === 'FERIAS')) {
                dtBloq.push(row.DTSERVPREST);
            }
        });

        return dtBloq;
    }

    public async abrirModalCriarLancRec() {
        this.modalCriarLancamentosRecorrentes?.destroy();
        this.modalCriarLancamentosRecorrentes = await ModalService.createFromJson('sm_cda/Modal/criar_lancamentos_recorrentes', true);
        this.modalCriarLancamentosRecorrentes.show();
    }

    public fecharModalCriarLancRec() {
        this.modalCriarLancamentosRecorrentes?.destroy();
        this.modalCriarLancamentosRecorrentes.hide();
    }

    public controleHabilitarOpcoes({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');
 
        const lancamentoVazio: any = grid.selectedRows.every((row: any) => (row.IDSERVPREST === null));

        if (lancamentoVazio) {
            component.datasource.data.forEach((elemento: any) => {
                elemento.isDisabled = false;
            });
        } else {
            component.datasource.data.forEach((elemento: any) => {
                if (elemento.value === 'ATIVIDADE_INTERNA') {
                    elemento.isDisabled = false;
                } else {
                    elemento.isDisabled = true;
                }
            });
        }
    }

    public controlaAcessoSelectJustificativa({ component }: any) {
        const selectJustificativa = Metadata.getInstance<Select>('SelectJustificativaLancamento');

        if (component.value === 'FALTA_JUSTIFICADA' || component.value === 'AFASTAMENTO_INSS') {
            selectJustificativa.isVisible = true;
            selectJustificativa.addValidation('required', {});
        } else {
            selectJustificativa.isVisible = false;
            selectJustificativa.removeValidation('required');
        }
    }

    public salvarLancamentosRecorrentes() {
        const form = Metadata.getInstance<Form>('form-criar-lancamentos-recorrentes');
        const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');

        if (Utilities.validateForm(form)) {
            LoadingService.show();

            const rows = grid.selectedRows;
            Http.post('/criarLancamentosRecorrentes', {
                ROWS: rows,
                NRMOTIVOEXCEFREQ: form.value.SelectJustificativaLancamento,
                IDSERVPREST: form.value.SelectTipoLancamento,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                grid.selectedRows = [];
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalCriarLancRec();
                LoadingService.hide();
            });
        }
    }

    public periodoAtual: string = '';

    public criaRecorrencia: boolean = false;
}
